#!/bin/sh

if grep -q QEMU /proc/cpuinfo || dmesg | grep -q 'Manufacturer: QEMU' || [ "${1}" = "force" ]; then
	for x in /dev/md/* ; do
		mdadm --stop $x
	done
	for disk in $(list-harddrives | grep -v fd0 | awk '{printf $1 " "}'); do
		wipefs -a /dev/$disk
		dd if=/dev/zero of=/dev/$disk bs=512 count=32
		parted -s /dev/${disk} mklabel gpt;
		partprobe
	done
fi

#MBR_LIMIT=2254857830400
MBR_LIMIT=21474836480
DISKS=""
RAIDS=""

prepare_serials() {
	local serial serial_scsi
	for disk in $(list-harddrives | grep -v fd0 | awk '{printf $1 " "}'); do
		serial=$(udevadm info --query=property --name=/dev/${disk} | grep ID_SERIAL_SHORT | cut -d= -f2)
		serial_scsi=$(udevadm info --query=property --name=/dev/${disk} | grep ID_SCSI_SERIAL | cut -d= -f2)
		if [ -n "${serial}" ]; then
			DISKS="${DISKS} ${serial}:/dev/${disk}"
		fi
		if [ -n "${serial_scsi}" ] && ! echo "${serial_scsi}" | grep -q '[[:space:]]'; then
			DISKS="${DISKS} ${serial_scsi}:/dev/${disk}"
		fi
	done
}

prepare_raid() {
	local line raidname raidlevel hdd hdds
	line="${1}"
	set ${line}
	while [ -n "${1}" ]; do
		name=${1%%=*}
		val=${1##*=}
		shift
		# echo "${name} ${val}"
		case ${name} in
			RAID_TYPE)
				raidlevel="${val##*_}"
			;;
			NAME)
				raidname="${val}"
			;;
			HDD_*)
				hdd=$(get_disk_device "${val}")
				if [ -n "${hdd}" ]; then
					if [ -n "${hdds}" ]; then
						hdds="${hdds}:${hdd}"
					else
						hdds="${hdd}"
					fi
				fi	
			;;
			DEV_*)
				hdd="${val}"
				if [ -n "${hdds}" ]; then
					hdds="${hdds}:${hdd}"
				else
					hdds="${hdd}"
				fi
			;;
		esac
	done
	RAIDS="${RAIDS} ${raidname}:${raidlevel}:${hdds}"
}

getval() {
	echo "${1##*=}"
}

do_part() {
	# $1 - hdd
	# $2 - part_string
	# echo $2
	local mpoint size fstype hdd
	hdd=${1}
	mpoint=$(echo "${2}" | cut -d, -f1)
	size=$(echo "${2}" | cut -d, -f2)
	fstype=$(echo "${2}" | cut -d, -f3)
	if [ "${mpoint}" = "swap" ]; then
		echo "part swap --size=${size} --asprimary --ondisk=${hdd}" | tee -a /tmp/part-include
	elif [ -z "${size}" ]; then
		echo "part ${mpoint} --fstype ${fstype} --size=1 --grow --asprimary --ondisk=${hdd}" | tee -a /tmp/part-include
	else
		echo "part ${mpoint} --fstype ${fstype} --size=${size} --asprimary --ondisk=${hdd}" | tee -a /tmp/part-include
	fi
}

do_raid_part() {
	# $1 - hdds
	# $2 - part_string
	# $3 - part_number
	# $4 - raid level
	# echo $2
	local mpoint size fstype hdds hdd pindex dindex raidparts raidpart raidlevel
	hdds="${1}"
	# echo "${hdds}"
	pindex="${3}"
	raidlevel="${4}"
	mpoint=$(echo "${2}" | cut -d, -f1)
	size=$(echo "${2}" | cut -d, -f2)
	fstype=$(echo "${2}" | cut -d, -f3)
	dindex=0
	for hdd in ${hdds}; do
		dindex=$((dindex + 1))
		raidpart="raid.${pindex}${dindex}"
		if [ -n "${size}" ]; then
			echo "part ${raidpart} --size=${size} --asprimary --ondisk=${hdd}" | tee -a /tmp/part-include
		else
			echo "part ${raidpart} --size=1 --grow --asprimary --ondisk=${hdd}" | tee -a /tmp/part-include
		fi
		raidparts="${raidparts} ${raidpart}"
	done
	echo "raid ${mpoint} --fstype ${fstype} --level=RAID${raidlevel} --device=md${pindex} ${raidparts}" | tee -a /tmp/part-include
}

get_disk_size() {
	fdisk -l "${1}" 2>/dev/null | grep Disk | grep dev | cut -d' ' -f5
}


get_disk_device() {
	local dev name serial
	name=${1}
	for part in ${DISKS}; do
		serial=${part%:*}	
		if [ "${serial}" = "${name}" ]; then
			dev=${part#*:}
			break
		fi
	done
	echo ${dev}
}

do_system_parts() {
	local hdd size
	hdd=${1}
	size=$(get_disk_size ${hdd})
	if [ -z "${size}" ]; then
		size=1
	fi
	if [ -d /sys/firmware/efi ]; then
		echo "part /boot/efi --fstype efi --size=250 --ondisk=${hdd}" | tee -a /tmp/part-include
    if [ ! -f /tmp/efi_first_part ]; then
        echo "${hdd}" >> /tmp/efi_first_part
    else
        echo "${hdd}" >> /tmp/efi_second_part
    fi
	elif [ ${size} -gt ${MBR_LIMIT} ]; then
		echo "part biosboot --fstype=biosboot --size=2 --ondisk=${hdd}" | tee -a /tmp/part-include
	fi
}

do_layout_hdd() {
	local line name val
	local dev install_os parts
	# echo "${line}"
	# echo ${@}
	while [ -n "${1}" ]; do
		name=${1%%=*}
		val=${1##*=}
		shift
		# echo "${name} ${val}"
		case ${name} in
			DEV)
				dev=${val}
			;;
			NAME)
				dev=$(get_disk_device "${val}")
			;;
			PART_*)
				part=${val#\"}
				part=${part%\"}
				#echo ${part}
				parts="${parts} ${part}"
			;;
			INSTALL_OS)
				install_os=true
			;;
		esac
	done
	if [ -n "${install_os}" ]; then
		do_system_parts ${dev}
	fi
	for part in ${parts} ; do
		do_part "${dev}" "${part}"
	done
}

get_raid_info() {
	local raidstr name lname
	name=${1}
	for raid in ${RAIDS} ; do
		lname=${raid%%:*}
		if [ "${lname}" = "${name}" ]; then
			raidstr=${raid#*:}
			break
		fi
	done
	echo "${raidstr}"
}

do_layout_raid() {
	local line name val
	local raid raidstr raidlevel hdds parts pindex
	# echo "${line}"
	# echo ${@}
	while [ -n "${1}" ]; do
		name=${1%%=*}
		val=${1##*=}
		shift
		# echo "${name} ${val}"
		case ${name} in
			NAME)
				raidstr=$(get_raid_info "${val}")
				raidlevel=${raidstr%%:*}
				hdds=$(echo "${raidstr#*:}" | sed 's|:| |g')
			;;
			PART_*)
				part=${val#\"}
				part=${part%\"}
				#echo ${part}
				parts="${parts} ${part}"
			;;
			INSTALL_OS)
				install_os=true
			;;
		esac
	done
	if [ -n "${install_os}" ]; then
		for dev in ${hdds}; do
			do_system_parts ${dev}
		done
	fi
	pindex=0
	for part in ${parts} ; do
		do_raid_part "${hdds}" "${part}" "${pindex}" "${raidlevel}"
		pindex=$((pindex + 1))
	done

}

do_layout() {
	local line valname
	line="${1}"
	set ${line}
	#echo ${1}
	#echo ${@}
	for val in ${line} ; do
		# echo ${val}
		valname=${val%%=*}
		# echo $valname
		case ${valname} in
			TYPE)
				type=$(getval ${val})
				line=$(echo "${line}" | sed "s|${val}||")
				break
		esac
	done
	case ${type} in
		hdd)
			do_layout_hdd ${line}
		;;
		raid)
			do_layout_raid ${line}
		;;
	esac
}

# Erase file
rm -f /tmp/part-include

if [ -d /sys/firmware/efi ]; then
	efibootmgr -v | grep BootCurrent | cut -d' ' -f2 > /tmp/efi_booted
fi

prepare_serials

if [ -f /tmp/diskpart.txt ]; then
    echo "" >> /tmp/diskpart.txt
fi
while read -r line ; do
	line_type=$(echo "${line}" | cut -d: -f1)
	line_content=$(echo "${line}" | cut -d: -f2-| sed 's|^\s||')
	case ${line_type} in
		raid)
			prepare_raid "${line_content}"
		;;
		layout)
			# Do layout
			do_layout "${line_content}"
		;;
	esac

done < /tmp/diskpart.txt

